@@ -1,11 +1,11 @@
 /* Pass translations to a subprocess.
-   Copyright (C) 2001-2006 Free Software Foundation, Inc.
+   Copyright (C) 2001-2010 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation; either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
 #ifdef HAVE_CONFIG_H
@@ -25,6 +24,7 @@
 #include <getopt.h>
 #include <limits.h>
 #include <locale.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,9 +40,12 @@
 #include "relocatable.h"
 #include "basename.h"
 #include "message.h"
+#include "read-catalog.h"
 #include "read-po.h"
+#include "read-properties.h"
+#include "read-stringtable.h"
+#include "msgl-charset.h"
 #include "xalloc.h"
-#include "exit.h"
 #include "full-write.h"
 #include "findprog.h"
 #include "pipe.h"
@@ -87,7 +90,7 @@ static const struct option long_options[] =
 /* Forward declaration of local functions.  */
 static void usage (int status)
 #if defined __GNUC__ && ((__GNUC__ == 2 && __GNUC_MINOR__ >= 5) || __GNUC__ > 2)
-	__attribute__ ((noreturn))
+        __attribute__ ((noreturn))
 #endif
 ;
 static void process_msgdomain_list (const msgdomain_list_ty *mdlp);
@@ -101,6 +104,7 @@ main (int argc, char **argv)
   bool do_version;
   const char *input_file;
   msgdomain_list_ty *result;
+  catalog_input_format_ty input_syntax = &input_format_po;
   size_t i;
 
   /* Set program name for messages.  */
@@ -128,44 +132,44 @@ main (int argc, char **argv)
   /* The '+' in the options string causes option parsing to terminate when
      the first non-option, i.e. the subprogram name, is encountered.  */
   while ((opt = getopt_long (argc, argv, "+D:hi:PV", long_options, NULL))
-	 != EOF)
+         != EOF)
     switch (opt)
       {
-      case '\0':		/* Long option.  */
-	break;
+      case '\0':                /* Long option.  */
+        break;
 
       case 'D':
-	dir_list_append (optarg);
-	break;
+        dir_list_append (optarg);
+        break;
 
       case 'h':
-	do_help = true;
-	break;
+        do_help = true;
+        break;
 
       case 'i':
-	if (input_file != NULL)
-	  {
-	    error (EXIT_SUCCESS, 0, _("at most one input file allowed"));
-	    usage (EXIT_FAILURE);
-	  }
-	input_file = optarg;
-	break;
+        if (input_file != NULL)
+          {
+            error (EXIT_SUCCESS, 0, _("at most one input file allowed"));
+            usage (EXIT_FAILURE);
+          }
+        input_file = optarg;
+        break;
 
       case 'P':
-	input_syntax = syntax_properties;
-	break;
+        input_syntax = &input_format_properties;
+        break;
 
       case 'V':
-	do_version = true;
-	break;
+        do_version = true;
+        break;
 
       case CHAR_MAX + 1: /* --stringtable-input */
-	input_syntax = syntax_stringtable;
-	break;
+        input_syntax = &input_format_stringtable;
+        break;
 
       default:
-	usage (EXIT_FAILURE);
-	break;
+        usage (EXIT_FAILURE);
+        break;
       }
 
   /* Version information is requested.  */
@@ -174,10 +178,11 @@ main (int argc, char **argv)
       printf ("%s (GNU %s) %s\n", basename (program_name), PACKAGE, VERSION);
       /* xgettext: no-wrap */
       printf (_("Copyright (C) %s Free Software Foundation, Inc.\n\
-This is free software; see the source for copying conditions.  There is NO\n\
-warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
+License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
+This is free software: you are free to change and redistribute it.\n\
+There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-	      "2001-2006");
+              "2001-2010");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -193,7 +198,7 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 
   /* Build argument list for the program.  */
   sub_argc = argc - optind;
-  sub_argv = (char **) xmalloc ((sub_argc + 1) * sizeof (char *));
+  sub_argv = XNMALLOC (sub_argc + 1, char *);
   for (i = 0; i < sub_argc; i++)
     sub_argv[i] = argv[optind + i];
   sub_argv[i] = NULL;
@@ -203,13 +208,28 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
     input_file = "-";
 
   /* Read input file.  */
-  result = read_po_file (input_file);
+  result = read_catalog_file (input_file, input_syntax);
 
   if (strcmp (sub_name, "0") != 0)
     {
+      /* Warn if the current locale is not suitable for this PO file.  */
+      compare_po_locale_charsets (result);
+
+      /* Block SIGPIPE for this process and for the subprocesses.
+         The subprogram may have side effects (additionally to producing some
+         output), therefore if there are no readers on stdout, processing of the
+         strings must continue nevertheless.  */
+      {
+        sigset_t sigpipe_set;
+
+        sigemptyset (&sigpipe_set);
+        sigaddset (&sigpipe_set, SIGPIPE);
+        sigprocmask (SIG_UNBLOCK, &sigpipe_set, NULL);
+      }
+
       /* Attempt to locate the program.
-	 This is an optimization, to avoid that spawn/exec searches the PATH
-	 on every call.  */
+         This is an optimization, to avoid that spawn/exec searches the PATH
+         on every call.  */
       sub_path = find_in_path (sub_name);
 
       /* Finish argument list for the program.  */
@@ -231,7 +251,7 @@ usage (int status)
 {
   if (status != EXIT_SUCCESS)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
+             program_name);
   else
     {
       printf (_("\
@@ -279,8 +299,12 @@ Informative output:\n"));
       printf (_("\
   -V, --version               output version information and exit\n"));
       printf ("\n");
+      /* TRANSLATORS: The placeholder indicates the bug-reporting address
+         for this package.  Please add _another line_ saying
+         "Report translation bugs to <...>\n" with the address for translation
+         bugs (typically your translation team's web or email address).  */
       fputs (_("Report bugs to <bug-gnu-gettext@gnu.org>.\n"),
-	     stdout);
+             stdout);
     }
 
   exit (status);
@@ -318,7 +342,7 @@ process_string (const message_ty *mp, const char *str, size_t len)
     {
       /* Built-in command "0".  */
       if (full_write (STDOUT_FILENO, str, len + 1) < len + 1)
-	error (EXIT_FAILURE, errno, _("write to stdout failed"));
+        error (EXIT_FAILURE, errno, _("write to stdout failed"));
     }
   else
     {
@@ -326,35 +350,55 @@ process_string (const message_ty *mp, const char *str, size_t len)
       char *location;
       pid_t child;
       int fd[1];
+      void (*orig_sigpipe_handler)(int);
       int exitstatus;
 
-      /* Set environment variables for the subprocess.  */
+      /* Set environment variables for the subprocess.
+         Note: These environment variables, especially MSGEXEC_MSGCTXT and
+         MSGEXEC_MSGCTXT, may contain non-ASCII characters.  The subprocess
+         may not interpret these values correctly if the locale encoding is
+         different from the PO file's encoding.  We want about this situation,
+         above.
+         On Unix, this problem is often harmless.  On Windows, however, - both
+         native Windows and Cygwin - the values of environment variables *must*
+         be in the encoding that is the value of GetACP(), because the system
+         may convert the environment from char** to wchar_t** before spawning
+         the subprocess and back from wchar_t** to char** in the subprocess,
+         and it does so using the GetACP() codepage.  */
       if (mp->msgctxt != NULL)
-	xsetenv ("MSGEXEC_MSGCTXT", mp->msgctxt, 1);
+        xsetenv ("MSGEXEC_MSGCTXT", mp->msgctxt, 1);
       else
-	unsetenv ("MSGEXEC_MSGCTXT");
+        unsetenv ("MSGEXEC_MSGCTXT");
       xsetenv ("MSGEXEC_MSGID", mp->msgid, 1);
       location = xasprintf ("%s:%ld", mp->pos.file_name,
-			    (long) mp->pos.line_number);
+                            (long) mp->pos.line_number);
       xsetenv ("MSGEXEC_LOCATION", location, 1);
       free (location);
 
       /* Open a pipe to a subprocess.  */
       child = create_pipe_out (sub_name, sub_path, sub_argv, NULL, false, true,
-			       true, fd);
+                               true, fd);
+
+      /* Ignore SIGPIPE here.  We don't care if the subprocesses terminates
+         successfully without having read all of the input that we feed it.  */
+      orig_sigpipe_handler = signal (SIGPIPE, SIG_IGN);
 
       if (full_write (fd[0], str, len) < len)
-	error (EXIT_FAILURE, errno,
-	       _("write to %s subprocess failed"), sub_name);
+        if (errno != EPIPE)
+          error (EXIT_FAILURE, errno,
+                 _("write to %s subprocess failed"), sub_name);
 
       close (fd[0]);
 
+      signal (SIGPIPE, orig_sigpipe_handler);
+
       /* Remove zombie process from process list, and retrieve exit status.  */
       /* FIXME: Should ignore_sigpipe be set to true here? It depends on the
-	 semantics of the subprogram...  */
-      exitstatus = wait_subprocess (child, sub_name, false, false, true, true);
+         semantics of the subprogram...  */
+      exitstatus =
+        wait_subprocess (child, sub_name, false, false, true, true, NULL);
       if (exitcode < exitstatus)
-	exitcode = exitstatus;
+        exitcode = exitstatus;
     }
 }
 
