@@ -1,5 +1,5 @@
 /* Edit translations using a subprocess.
-   Copyright (C) 2001-2007 Free Software Foundation, Inc.
+   Copyright (C) 2001-2010 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software: you can redistribute it and/or modify
@@ -20,8 +20,6 @@
 # include "config.h"
 #endif
 
-#include <errno.h>
-#include <fcntl.h>
 #include <getopt.h>
 #include <limits.h>
 #include <locale.h>
@@ -31,18 +29,11 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <unistd.h>
-#if defined _MSC_VER || defined __MINGW32__
-# include <io.h>
-#endif
-
-/* Get fd_set (on AIX or Minix) or select() declaration (on EMX).  */
-#if defined (_AIX) || defined (_MINIX) || defined (__EMX__)
-# include <sys/select.h>
-#endif
 
 #include "closeout.h"
 #include "dir-list.h"
 #include "error.h"
+#include "xvasprintf.h"
 #include "error-progname.h"
 #include "progname.h"
 #include "relocatable.h"
@@ -56,11 +47,12 @@
 #include "write-po.h"
 #include "write-properties.h"
 #include "write-stringtable.h"
+#include "color.h"
 #include "msgl-charset.h"
 #include "xalloc.h"
 #include "findprog.h"
-#include "pipe.h"
-#include "wait-process.h"
+#include "pipe-filter.h"
+#include "xsetenv.h"
 #include "filters.h"
 #include "msgl-iconv.h"
 #include "po-charset.h"
@@ -70,17 +62,7 @@
 #define _(str) gettext (str)
 
 
-/* We use a child process, and communicate through a bidirectional pipe.
-   To avoid deadlocks, let the child process decide when it wants to read
-   or to write, and let the parent behave accordingly.  The parent uses
-   select() to know whether it must write or read.  On platforms without
-   select(), we use non-blocking I/O.  (This means the parent is busy
-   looping while waiting for the child.  Not good.)  */
-
-/* On BeOS select() works only on sockets, not on normal file descriptors.  */
-#ifdef __BEOS__
-# undef HAVE_SELECT
-#endif
+/* We use a child process, and communicate through a bidirectional pipe.  */
 
 
 /* Force output of PO file even if empty.  */
@@ -96,7 +78,7 @@ static const char *sub_name;
 static const char *sub_path;
 
 /* Argument list for the subprogram.  */
-static char **sub_argv;
+static const char **sub_argv;
 static int sub_argc;
 
 /* Filter function.  */
@@ -106,6 +88,7 @@ static void (*filter) (const char *str, size_t len, char **resultp, size_t *leng
 static const struct option long_options[] =
 {
   { "add-location", no_argument, &line_comment, 1 },
+  { "color", optional_argument, NULL, CHAR_MAX + 6 },
   { "directory", required_argument, NULL, 'D' },
   { "escape", no_argument, NULL, 'E' },
   { "force-po", no_argument, &force_po, 1 },
@@ -124,6 +107,7 @@ static const struct option long_options[] =
   { "strict", no_argument, NULL, 'S' },
   { "stringtable-input", no_argument, NULL, CHAR_MAX + 4 },
   { "stringtable-output", no_argument, NULL, CHAR_MAX + 5 },
+  { "style", required_argument, NULL, CHAR_MAX + 7 },
   { "version", no_argument, NULL, 'V' },
   { "width", required_argument, NULL, 'w', },
   { NULL, 0, NULL, 0 }
@@ -133,7 +117,7 @@ static const struct option long_options[] =
 /* Forward declaration of local functions.  */
 static void usage (int status)
 #if defined __GNUC__ && ((__GNUC__ == 2 && __GNUC_MINOR__ >= 5) || __GNUC__ > 2)
-	__attribute__ ((noreturn))
+        __attribute__ ((noreturn))
 #endif
 ;
 static void generic_filter (const char *str, size_t len, char **resultp, size_t *lengthp);
@@ -181,95 +165,104 @@ main (int argc, char **argv)
   /* The '+' in the options string causes option parsing to terminate when
      the first non-option, i.e. the subprogram name, is encountered.  */
   while ((opt = getopt_long (argc, argv, "+D:EFhi:o:pPsVw:", long_options,
-			     NULL))
-	 != EOF)
+                             NULL))
+         != EOF)
     switch (opt)
       {
-      case '\0':		/* Long option.  */
-	break;
+      case '\0':                /* Long option.  */
+        break;
 
       case 'D':
-	dir_list_append (optarg);
-	break;
+        dir_list_append (optarg);
+        break;
 
       case 'E':
-	message_print_style_escape (true);
-	break;
+        message_print_style_escape (true);
+        break;
 
       case 'F':
-	sort_by_filepos = true;
-	break;
+        sort_by_filepos = true;
+        break;
 
       case 'h':
-	do_help = true;
-	break;
+        do_help = true;
+        break;
 
       case 'i':
-	if (input_file != NULL)
-	  {
-	    error (EXIT_SUCCESS, 0, _("at most one input file allowed"));
-	    usage (EXIT_FAILURE);
-	  }
-	input_file = optarg;
-	break;
+        if (input_file != NULL)
+          {
+            error (EXIT_SUCCESS, 0, _("at most one input file allowed"));
+            usage (EXIT_FAILURE);
+          }
+        input_file = optarg;
+        break;
 
       case 'o':
-	output_file = optarg;
-	break;
+        output_file = optarg;
+        break;
 
       case 'p':
-	output_syntax = &output_format_properties;
-	break;
+        output_syntax = &output_format_properties;
+        break;
 
       case 'P':
-	input_syntax = &input_format_properties;
-	break;
+        input_syntax = &input_format_properties;
+        break;
 
       case 's':
-	sort_by_msgid = true;
-	break;
+        sort_by_msgid = true;
+        break;
 
       case 'S':
-	message_print_style_uniforum ();
-	break;
+        message_print_style_uniforum ();
+        break;
 
       case 'V':
-	do_version = true;
-	break;
+        do_version = true;
+        break;
 
       case 'w':
-	{
-	  int value;
-	  char *endp;
-	  value = strtol (optarg, &endp, 10);
-	  if (endp != optarg)
-	    message_page_width_set (value);
-	}
-	break;
+        {
+          int value;
+          char *endp;
+          value = strtol (optarg, &endp, 10);
+          if (endp != optarg)
+            message_page_width_set (value);
+        }
+        break;
 
       case CHAR_MAX + 1:
-	message_print_style_indent ();
-	break;
+        message_print_style_indent ();
+        break;
 
       case CHAR_MAX + 2:
-	message_print_style_escape (false);
-	break;
+        message_print_style_escape (false);
+        break;
 
       case CHAR_MAX + 3: /* --no-wrap */
-	message_page_width_ignore ();
-	break;
+        message_page_width_ignore ();
+        break;
 
       case CHAR_MAX + 4: /* --stringtable-input */
-	input_syntax = &input_format_stringtable;
-	break;
+        input_syntax = &input_format_stringtable;
+        break;
 
       case CHAR_MAX + 5: /* --stringtable-output */
-	output_syntax = &output_format_stringtable;
-	break;
+        output_syntax = &output_format_stringtable;
+        break;
+
+      case CHAR_MAX + 6: /* --color */
+        if (handle_color_option (optarg) || color_test_mode)
+          usage (EXIT_FAILURE);
+        break;
+
+      case CHAR_MAX + 7: /* --style */
+        handle_style_option (optarg);
+        break;
 
       default:
-	usage (EXIT_FAILURE);
-	break;
+        usage (EXIT_FAILURE);
+        break;
       }
 
   /* Version information is requested.  */
@@ -282,7 +275,7 @@ License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n\
 "),
-	      "2001-2007");
+              "2001-2010");
       printf (_("Written by %s.\n"), proper_name ("Bruno Haible"));
       exit (EXIT_SUCCESS);
     }
@@ -299,15 +292,15 @@ There is NO WARRANTY, to the extent permitted by law.\n\
   /* Verify selected options.  */
   if (!line_comment && sort_by_filepos)
     error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
-	   "--no-location", "--sort-by-file");
+           "--no-location", "--sort-by-file");
 
   if (sort_by_msgid && sort_by_filepos)
     error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
-	   "--sort-output", "--sort-by-file");
+           "--sort-output", "--sort-by-file");
 
   /* Build argument list for the program.  */
   sub_argc = argc - optind;
-  sub_argv = XNMALLOC (sub_argc + 1, char *);
+  sub_argv = XNMALLOC (sub_argc + 1, const char *);
   for (i = 0; i < sub_argc; i++)
     sub_argv[i] = argv[optind + i];
   sub_argv[i] = NULL;
@@ -316,24 +309,24 @@ There is NO WARRANTY, to the extent permitted by law.\n\
   if (strcmp (sub_name, "sed") == 0)
     {
       if (sub_argc == 1)
-	error (EXIT_FAILURE, 0,
-	       _("at least one sed script must be specified"));
+        error (EXIT_FAILURE, 0,
+               _("at least one sed script must be specified"));
 
       /* Replace GNU sed specific options with portable sed options.  */
       for (i = 1; i < sub_argc; i++)
-	{
-	  if (strcmp (sub_argv[i], "--expression") == 0)
-	    sub_argv[i] = "-e";
-	  else if (strcmp (sub_argv[i], "--file") == 0)
-	    sub_argv[i] = "-f";
-	  else if (strcmp (sub_argv[i], "--quiet") == 0
-		   || strcmp (sub_argv[i], "--silent") == 0)
-	    sub_argv[i] = "-n";
+        {
+          if (strcmp (sub_argv[i], "--expression") == 0)
+            sub_argv[i] = "-e";
+          else if (strcmp (sub_argv[i], "--file") == 0)
+            sub_argv[i] = "-f";
+          else if (strcmp (sub_argv[i], "--quiet") == 0
+                   || strcmp (sub_argv[i], "--silent") == 0)
+            sub_argv[i] = "-n";
 
-	  if (strcmp (sub_argv[i], "-e") == 0
-	      || strcmp (sub_argv[i], "-f") == 0)
-	    i++;
-	}
+          if (strcmp (sub_argv[i], "-e") == 0
+              || strcmp (sub_argv[i], "-f") == 0)
+            i++;
+        }
     }
 
   /* By default, input comes from standard input.  */
@@ -359,12 +352,12 @@ There is NO WARRANTY, to the extent permitted by law.\n\
       compare_po_locale_charsets (result);
 
       /* Attempt to locate the program.
-	 This is an optimization, to avoid that spawn/exec searches the PATH
-	 on every call.  */
+         This is an optimization, to avoid that spawn/exec searches the PATH
+         on every call.  */
       sub_path = find_in_path (sub_name);
 
       /* Finish argument list for the program.  */
-      sub_argv[0] = (char *) sub_path;
+      sub_argv[0] = sub_path;
     }
 
   /* Apply the subprogram.  */
@@ -389,7 +382,7 @@ usage (int status)
 {
   if (status != EXIT_SUCCESS)
     fprintf (stderr, _("Try `%s --help' for more information.\n"),
-	     program_name);
+             program_name);
   else
     {
       printf (_("\
@@ -444,6 +437,12 @@ Input file syntax:\n"));
       printf ("\n");
       printf (_("\
 Output details:\n"));
+      printf (_("\
+      --color                 use colors and other text attributes always\n\
+      --color=WHEN            use colors and other text attributes if WHEN.\n\
+                              WHEN may be 'always', 'never', 'auto', or 'html'.\n"));
+      printf (_("\
+      --style=STYLEFILE       specify CSS style rule file for --color\n"));
       printf (_("\
       --no-escape             do not use C escapes in output (default)\n"));
       printf (_("\
@@ -486,234 +485,92 @@ Informative output:\n"));
          "Report translation bugs to <...>\n" with the address for translation
          bugs (typically your translation team's web or email address).  */
       fputs (_("Report bugs to <bug-gnu-gettext@gnu.org>.\n"),
-	     stdout);
+             stdout);
     }
 
   exit (status);
 }
 
 
-#ifdef EINTR
+/* Callbacks called from pipe_filter_ii_execute.  */
 
-/* EINTR handling for close(), read(), write(), select().
-   These functions can return -1/EINTR even though we don't have any
-   signal handlers set up, namely when we get interrupted via SIGSTOP.  */
-
-static inline int
-nonintr_close (int fd)
+struct locals
 {
-  int retval;
+  /* String being written.  */
+  const char *str;
+  size_t len;
+  /* String being read and accumulated.  */
+  char *result;
+  size_t allocated;
+  size_t length;
+};
 
-  do
-    retval = close (fd);
-  while (retval < 0 && errno == EINTR);
-
-  return retval;
-}
-#define close nonintr_close
-
-static inline ssize_t
-nonintr_read (int fd, void *buf, size_t count)
+static const void *
+prepare_write (size_t *num_bytes_p, void *private_data)
 {
-  ssize_t retval;
+  struct locals *l = (struct locals *) private_data;
 
-  do
-    retval = read (fd, buf, count);
-  while (retval < 0 && errno == EINTR);
-
-  return retval;
+  if (l->len > 0)
+    {
+      *num_bytes_p = l->len;
+      return l->str;
+    }
+  else
+    return NULL;
 }
-#define read nonintr_read
 
-static inline ssize_t
-nonintr_write (int fd, const void *buf, size_t count)
+static void
+done_write (void *data_written, size_t num_bytes_written, void *private_data)
 {
-  ssize_t retval;
+  struct locals *l = (struct locals *) private_data;
 
-  do
-    retval = write (fd, buf, count);
-  while (retval < 0 && errno == EINTR);
-
-  return retval;
+  l->str += num_bytes_written;
+  l->len -= num_bytes_written;
 }
-#undef write /* avoid warning on VMS */
-#define write nonintr_write
 
-# if HAVE_SELECT
-
-static inline int
-nonintr_select (int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds,
-		struct timeval *timeout)
+static void *
+prepare_read (size_t *num_bytes_p, void *private_data)
 {
-  int retval;
+  struct locals *l = (struct locals *) private_data;
 
-  do
-    retval = select (n, readfds, writefds, exceptfds, timeout);
-  while (retval < 0 && errno == EINTR);
-
-  return retval;
+  if (l->length == l->allocated)
+    {
+      l->allocated = l->allocated + (l->allocated >> 1);
+      l->result = (char *) xrealloc (l->result, l->allocated);
+    }
+  *num_bytes_p = l->allocated - l->length;
+  return l->result + l->length;
 }
-#undef select /* avoid warning on VMS */
-#define select nonintr_select
 
-# endif
+static void
+done_read (void *data_read, size_t num_bytes_read, void *private_data)
+{
+  struct locals *l = (struct locals *) private_data;
 
-#endif
+  l->length += num_bytes_read;
+}
 
 
-/* Non-blocking I/O.  */
-#ifndef O_NONBLOCK
-# define O_NONBLOCK O_NDELAY
-#endif
-#if HAVE_SELECT
-# define IS_EAGAIN(errcode) 0
-#else
-# ifdef EWOULDBLOCK
-#  define IS_EAGAIN(errcode) ((errcode) == EAGAIN || (errcode) == EWOULDBLOCK)
-# else
-#  define IS_EAGAIN(errcode) ((errcode) == EAGAIN)
-# endif
-#endif
-
 /* Process a string STR of size LEN bytes through the subprogram.
    Store the freshly allocated result at *RESULTP and its length at *LENGTHP.
  */
 static void
 generic_filter (const char *str, size_t len, char **resultp, size_t *lengthp)
 {
-#if defined _MSC_VER || defined __MINGW32__
-  /* Native Woe32 API.  */
-  /* Not yet implemented.  */
-  error (EXIT_FAILURE, 0, _("Not yet implemented."));
-#else
-  pid_t child;
-  int fd[2];
-  char *result;
-  size_t allocated;
-  size_t length;
-  int exitstatus;
+  struct locals l;
 
-  /* Open a bidirectional pipe to a subprocess.  */
-  child = create_pipe_bidi (sub_name, sub_path, sub_argv, false, true, true,
-			    fd);
+  l.str = str;
+  l.len = len;
+  l.allocated = len + (len >> 2) + 1;
+  l.result = XNMALLOC (l.allocated, char);
+  l.length = 0;
 
-  /* Enable non-blocking I/O.  This permits the read() and write() calls
-     to return -1/EAGAIN without blocking; this is important for polling
-     if HAVE_SELECT is not defined.  It also permits the read() and write()
-     calls to return after partial reads/writes; this is important if
-     HAVE_SELECT is defined, because select() only says that some data
-     can be read or written, not how many.  Without non-blocking I/O,
-     Linux 2.2.17 and BSD systems prefer to block instead of returning
-     with partial results.  */
-  {
-    int fcntl_flags;
+  pipe_filter_ii_execute (sub_name, sub_path, sub_argv, false, true,
+                          prepare_write, done_write, prepare_read, done_read,
+                          &l);
 
-    if ((fcntl_flags = fcntl (fd[1], F_GETFL, 0)) < 0
-	|| fcntl (fd[1], F_SETFL, fcntl_flags | O_NONBLOCK) < 0
-	|| (fcntl_flags = fcntl (fd[0], F_GETFL, 0)) < 0
-	|| fcntl (fd[0], F_SETFL, fcntl_flags | O_NONBLOCK) < 0)
-      error (EXIT_FAILURE, errno,
-	     _("cannot set up nonblocking I/O to %s subprocess"), sub_name);
-  }
-
-  allocated = len + (len >> 2) + 1;
-  result = XNMALLOC (allocated, char);
-  length = 0;
-
-  for (;;)
-    {
-#if HAVE_SELECT
-      int n;
-      fd_set readfds;
-      fd_set writefds;
-
-      FD_ZERO (&readfds);
-      FD_SET (fd[0], &readfds);
-      n = fd[0] + 1;
-      if (str != NULL)
-	{
-	  FD_ZERO (&writefds);
-	  FD_SET (fd[1], &writefds);
-	  if (n <= fd[1])
-	    n = fd[1] + 1;
-	}
-
-      n = select (n, &readfds, (str != NULL ? &writefds : NULL), NULL, NULL);
-      if (n < 0)
-	error (EXIT_FAILURE, errno,
-	       _("communication with %s subprocess failed"), sub_name);
-      if (str != NULL && FD_ISSET (fd[1], &writefds))
-	goto try_write;
-      if (FD_ISSET (fd[0], &readfds))
-	goto try_read;
-      /* How could select() return if none of the two descriptors is ready?  */
-      abort ();
-#endif
-
-      /* Attempt to write.  */
-#if HAVE_SELECT
-    try_write:
-#endif
-      if (str != NULL)
-	{
-	  if (len > 0)
-	    {
-	      ssize_t nwritten = write (fd[1], str, len);
-	      if (nwritten < 0 && !IS_EAGAIN (errno))
-		error (EXIT_FAILURE, errno,
-		       _("write to %s subprocess failed"), sub_name);
-	      if (nwritten > 0)
-		{
-		  str += nwritten;
-		  len -= nwritten;
-		}
-	    }
-	  else
-	    {
-	      /* Tell the child there is nothing more the parent will send.  */
-	      close (fd[1]);
-	      str = NULL;
-	    }
-	}
-#if HAVE_SELECT
-      continue;
-#endif
-
-      /* Attempt to read.  */
-#if HAVE_SELECT
-    try_read:
-#endif
-      if (length == allocated)
-	{
-	  allocated = allocated + (allocated >> 1);
-	  result = (char *) xrealloc (result, allocated);
-	}
-      {
-	ssize_t nread = read (fd[0], result + length, allocated - length);
-	if (nread < 0 && !IS_EAGAIN (errno))
-	  error (EXIT_FAILURE, errno,
-		 _("read from %s subprocess failed"), sub_name);
-	if (nread > 0)
-	  length += nread;
-	if (nread == 0 && str == NULL)
-	  break;
-      }
-#if HAVE_SELECT
-      continue;
-#endif
-    }
-
-  close (fd[0]);
-
-  /* Remove zombie process from process list.  */
-  exitstatus = wait_subprocess (child, sub_name, false, false, true, true);
-  if (exitstatus != 0)
-    error (EXIT_FAILURE, 0, _("%s subprocess terminated with exit code %d"),
-	   sub_name, exitstatus);
-
-  *resultp = result;
-  *lengthp = length;
-#endif
+  *resultp = l.result;
+  *lengthp = l.length;
 }
 
 
@@ -735,16 +592,16 @@ process_string (const char *str, size_t len, char **resultp, size_t *lengthp)
 
     for (; p < pend; p++)
       if (*p == '\0')
-	{
-	  char *q;
+        {
+          char *q;
 
-	  q = p;
-	  for (; p < pend; p++)
-	    if (*p != '\0')
-	      *q++ = *p;
-	  length = q - result;
-	  break;
-	}
+          q = p;
+          for (; p < pend; p++)
+            if (*p != '\0')
+              *q++ = *p;
+          length = q - result;
+          break;
+        }
   }
 
   *resultp = result;
@@ -757,6 +614,7 @@ process_message (message_ty *mp)
 {
   const char *msgstr = mp->msgstr;
   size_t msgstr_len = mp->msgstr_len;
+  char *location;
   size_t nsubstrings;
   char **substrings;
   size_t total_len;
@@ -769,6 +627,28 @@ process_message (message_ty *mp)
   if (is_header (mp) && keep_header)
     return;
 
+  /* Set environment variables for the subprocess.
+     Note: These environment variables, especially MSGEXEC_MSGCTXT and
+     MSGEXEC_MSGCTXT, may contain non-ASCII characters.  The subprocess
+     may not interpret these values correctly if the locale encoding is
+     different from the PO file's encoding.  We want about this situation,
+     above.
+     On Unix, this problem is often harmless.  On Windows, however, - both
+     native Windows and Cygwin - the values of environment variables *must*
+     be in the encoding that is the value of GetACP(), because the system
+     may convert the environment from char** to wchar_t** before spawning
+     the subprocess and back from wchar_t** to char** in the subprocess,
+     and it does so using the GetACP() codepage.  */
+  if (mp->msgctxt != NULL)
+    xsetenv ("MSGFILTER_MSGCTXT", mp->msgctxt, 1);
+  else
+    unsetenv ("MSGFILTER_MSGCTXT");
+  xsetenv ("MSGFILTER_MSGID", mp->msgid, 1);
+  location = xasprintf ("%s:%ld", mp->pos.file_name,
+                        (long) mp->pos.line_number);
+  xsetenv ("MSGFILTER_LOCATION", location, 1);
+  free (location);
+
   /* Count NUL delimited substrings.  */
   for (p = msgstr, nsubstrings = 0;
        p < msgstr + msgstr_len;
